//Spirale3D_401f.js
// 27.01.14 / Hans Haesler, Chtelard 52, CH-1018 Lausanne
//DESCRIPTION:Dessiner une spirale en 3D

// activer l'affichage des dialogues
app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll;

// tester s'il y a un document ouvert
if (app.documents.length == 0) {
	alert ("Il n'y a pas de document ouvert.", "Attention");
	exit();
}

// tester, s'il y a un objet slectionn
if (app.selection.length == 0) {
	alert ("Slectionner un bloc, s.v.p.", "Attention");
	exit();
}

// le document au premier plan
var curDoc = app.documents[0];

// mmoriser les units de mesure, l'origine des rgles, les points zro et de rfrence
hM = curDoc.viewPreferences.horizontalMeasurementUnits;
vM = curDoc.viewPreferences.verticalMeasurementUnits;
rO = curDoc.viewPreferences.rulerOrigin;
zP = curDoc.zeroPoint;
tR = curDoc.layoutWindows[0].transformReferencePoint;

// modifier les rglages
curDoc.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.millimeters;
curDoc.viewPreferences.verticalMeasurementUnits = MeasurementUnits.millimeters;
curDoc.viewPreferences.rulerOrigin = RulerOrigin.SPREAD_ORIGIN;
curDoc.zeroPoint = [0,0];
curDoc.layoutWindows[0].transformReferencePoint = AnchorPoint.leftCenterAnchor;

// mmoriser la dimension de la slection
try {
	var gB = app.selection[0].geometricBounds;
}
catch (e) {
	alert ("Il faudrait slectionner un bloc rectangulaire.", "Stop");
	reStoreSettings();
	exit();
}

// appeler la fonction qui lit les prfrences mmorises
var allPrefs = getPrefs();
ck01 = ck02 = 0;
if (allPrefs != "") {
	var prefVal = allPrefs.split("\n");
	strW = prefVal[0]*1;
	strT = prefVal[1]*1;
	numV = prefVal[2]*1;
	pcHg = prefVal[3]*1;
	pcWd = prefVal[4]*1;
	rb01 = prefVal[5]*1;
	if (rb01 == 0) {
		ck01 = 1;
	}
	else {
		ck02 = 1;
	}
}
else {
	// sinon utiliser les valeurs par dfaut 
	// quand le fichier des prfrences n'existe pas
	strW = 1;
	strT = 100;
	numV = 10;
	pcHg = 60;
	pcWd = 90;
	ck01 = 1;
	ck02 = 0;
}

// appeler la fonction qui affiche le dialogue et qui renvoie le rsultat
var anArray = displayDialog();
var strW    = anArray[0];
var strT    = anArray[1];
var numVal  = anArray[2];
var pcHeig  = anArray[3];
var pcWidt  = anArray[4];
var dirDec  = anArray[5];

// les positions y1, x1, x2
var y1 = (Math.floor((gB[0]*1000)+0.5))/1000;
var x1 = (Math.floor((gB[1]*1000)+0.5))/1000;
var y2 = (Math.floor((gB[2]*1000)+0.5))/1000;
var x2 = (Math.floor((gB[3]*1000)+0.5))/1000;

// le milieu horizontal de l'objet
var m1 = x1 + ((x2-x1)/2);

// calculer le nombre de tours
var nRounds = (Math.ceil(numVal/2))-1;
	
// prparer une liste
var allFrames = new Array ();

// crer un bloc ovale...
var baseOne = app.activeWindow.activePage.ovals.add();
baseOne.properties = {geometricBounds:[y1, x1, y2, x2], fillColor:"None", strokeColor:"None"};
//... et l'ajouter  la liste
allFrames.push(baseOne);

// ouvrir le contour
baseOne.paths[0].pathType = PathType.OPEN_PATH;

// supprimer le premier point d'ancrage; le rsultat: un demi-arc
baseOne.paths[0].pathPoints[3].remove();

if (numVal > 1) {
	// calculer la position du demi-arc  gauche
	var yb1 = y1 + (((y2-y1)/100)*(100-pcHeig));
	
	// dupliquer le demi-arc...
	var baseTwo = baseOne.duplicate(undefined, [0, 0]);
	// ... et l'ajouter  la liste
	allFrames.push(baseTwo);
	
	// miroiter la copie
	baseTwo.flipItem(Flip.HORIZONTAL);
	
	// modifier la copie verticalement
	var gB = baseTwo.geometricBounds;
	var xb1 = gB[1];
	var yb2 = gB[2];
	baseTwo.geometricBounds = [yb1, xb1, yb2, m1];
	
	var frameA = baseOne;
	var frameB = baseTwo;
	var ya1 = yb1;
	
	// dupliquer les demi-arcs de base dans une boucle et les modifier
	for (var n=0; n<nRounds; n++) {
	
		// dupliquer le premier demi-arc...
		var frameA = frameA.duplicate([m1, ya1]);
		// ... et l'ajouter  la liste
		allFrames.push(frameA);
		
		// modifier la copie
		var gB = frameA.geometricBounds;
		ya1 = gB[0];
		xa2 = m1+((gB[3]-m1)/100)*pcWidt;
		ya2 = ya1+((gB[2]-ya1)/100)*pcWidt;
		frameA.geometricBounds = [ya1, m1, ya2, xa2];
		
		// sortir de la boucle avant le dernier lors 
		if (numVal%2 == 1) {
			if (n == nRounds-1) {
				break;
			}
		}
		 
		try {
			// dupliquer le deuxime demi-arc
			var frameB = frameB.duplicate([m1, ya2-(yb2-yb1)]);
			// modifier la copie ...
			var gB = frameB.geometricBounds;
			yb1 = gB[2]-((gB[2]-gB[0])/100)*pcWidt;
			xb1 = m1-((m1-gB[1])/100)*pcWidt;
			yb2 = gB[2];
			frameB.geometricBounds = [yb1, xb1, yb2, m1];
			// ... et l'ajouter  la liste
			allFrames.push(frameB);
			var ya1 = yb1;
		}
		catch (e) {
			frameA.remove();
		}
	}
	
	// joindre les points d'extrmit
	for (var n=1; n<allFrames.length; n++) {
		if (n%2 == 0) {
			try {
				allFrames[0].paths[0].pathPoints[0].join(allFrames[n].paths[0].pathPoints[2], JoinOptions.COMBINE);
			}
			catch (e) {
			}
		}
		else {
			try {
				allFrames[0].paths[0].pathPoints[0].join(allFrames[n].paths[0].pathPoints[0], JoinOptions.COMBINE);
			}
			catch (e) {
			}
		}
	}
}

// appliquer quelques attributs au contour
allFrames[0].appliedObjectStyle = curDoc.objectStyles.item(0);
allFrames[0].properties = {strokeColor:"Black", strokeWeight:strW, strokeTint:strT, 
	strokeAlignment:StrokeAlignment.CENTER_ALIGNMENT, endCap:EndCap.ROUND_END_CAP};

// inverser la direction
allFrames[0].paths[0].reverse ();

// "dans le sens des aiguilles d'une montre" (= par dfaut) ou "dans le sens inverse des aiguilles d'une montre"
if (dirDec == 0) {
	var dirTxt = "dans le sens des aiguilles d'une montre";
}
else if (dirDec == 1) {
	var dirTxt = "dans le sens inverse des aiguilles d'une montre";
	// slectionner le point de rfrence au milieu
	curDoc.layoutWindows[0].transformReferencePoint = AnchorPoint.centerAnchor;
	// "symtrie axe horizontal" de la spirale
	allFrames[0].flipItem(Flip.HORIZONTAL);
}

// entrer le rsultat dans l'tiquette
allFrames[0].label = " le nombre de demi-arcs: " + numVal + "\r\n la hauteur des arcs gauche: " + pcHeig + "%\r\n modification de la largeur: " + pcWidt +"%\r\n " + dirTxt;

// dslectionner
app.selection = null;

// appeler la fonction pour restaurer les rglages
reStoreSettings();


////////////////////////////// les fonctions //////////////////////////////////////////////////////

// la fonction qui renvoie les valeurs mmorises  ////////////////////////////////////////////////
function getPrefs() {
	var allPrefs = "";
	// lire le fichier qui contient les valeurs mmorises
	if ($.os.slice(0,7) == "Windows") {
		var tempFolder = Folder.temp;
		var f = File(tempFolder + "/Spirale3D401.txt");
	}
	else {
		var f = new File("~/Library/Preferences/Spirale3D401.txt");
	}
	var fileIsOpen = f.open("r");
	if (fileIsOpen) {
		try {
			allPrefs = f.read();
			f.close();
		}
		catch (e) {
			try {
				f.close();
			}
			catch (e) {
			}
		}
	}
	return allPrefs;
}

// prparer et afficher le dialogue ///////////////////////////////////////////////////////////////
function displayDialog() {
	try {
		app.dialogs.everyItem().destroy();
	}
	catch (e) {
	}
	var labelWidth = 155;
	var fieldWidth =  36;
	var aDialog = app.dialogs.add({name:"Spirale3D"});
	with (aDialog) {
		with (dialogColumns.add()) {
			with (borderPanels.add()) {
				with (dialogColumns.add()) {
					with (dialogRows.add()) {
					staticTexts.add({staticLabel:"L'paisseur du contour:", minWidth:labelWidth});
						var weightField = realEditboxes.add({editValue:strW, minimumValue:0.1, maximumValue:99, minWidth:fieldWidth});
						staticTexts.add({staticLabel:"pt", minWidth:12});
					}
					with (dialogRows.add()) {
						staticTexts.add({staticLabel:"La teinte du contour:", minWidth:labelWidth});
						var tintField = realEditboxes.add({editValue:strT, minimumValue:1, maximumValue:100, minWidth:fieldWidth});
						staticTexts.add({staticLabel:"%", minWidth:12});
						staticTexts.add({staticLabel:"", minWidth:0});
					}
				}
			}
			with (borderPanels.add()) {
				with (dialogColumns.add()) {
					with (dialogRows.add()) {
						staticTexts.add({staticLabel:"Le nombre de demi-arcs:", minWidth:labelWidth});
						var numField = integerEditboxes.add({editValue:numV, minimumValue:1, maximumValue:123, minWidth:fieldWidth});
						staticTexts.add({staticLabel:"", minWidth:18});
					}
				}
			}
			with (borderPanels.add()) {
				with (dialogColumns.add()) {
					with (dialogRows.add()) {
						staticTexts.add({staticLabel:"Hauteur des arcs  gauche:", minWidth:labelWidth});
						var pcHeightField = realEditboxes.add({editValue:pcHg, minimumValue:0.1, maximumValue:300, minWidth:fieldWidth});
						staticTexts.add({staticLabel:"%", minWidth:8});
						staticTexts.add({staticLabel:"", minWidth:0});
					}
				}
			}
			with (borderPanels.add()) {
				with (dialogColumns.add()) {
					with (dialogRows.add()) {
						staticTexts.add({staticLabel:"Modifier la largeur..."});
					}
					with (dialogRows.add()) {
						staticTexts.add({staticLabel:"... aprs chaque tour :", minWidth:labelWidth});
						var pcWidthField = realEditboxes.add({editValue:pcWd, minimumValue:50, maximumValue:150, minWidth:fieldWidth});
						staticTexts.add({staticLabel:"%", minWidth:8});
						staticTexts.add({staticLabel:"", minWidth:0});
					}
				}
			}
			with (borderPanels.add()) {
				with (dialogColumns.add()) {
					with (dialogRows.add()) {
						staticTexts.add({staticLabel:"Le contour dans le sens...", minWidth:75});
						staticTexts.add({staticLabel:" ", minWidth:71});
					}
					with (dialogRows.add()) {
						with(dirButtons = radiobuttonGroups.add()){
							radiobuttonControls.add({staticLabel:"des aiguilles d'une montre", checkedState:ck01});
							radiobuttonControls.add({staticLabel:"inverse des aiguilles d'une montre", checkedState:ck02});
						}
					}
				}
			}
		}
	}
	if(aDialog.show() == true) {
		// relever la saisie
		var strWght = weightField.editValue;
		var strTint = tintField.editValue;
		var numVal  = numField.editValue;
		var pcHeigh = pcHeightField.editValue;
		var pcWidth = pcWidthField.editValue;
		var dirDec  = dirButtons.selectedButton;
		// supprimer le dialogue
		aDialog.destroy();
		// la nouvelle liste des prfrences
		var prefsList = new Array (strWght, strTint, numVal, pcHeigh, pcWidth, dirDec);
		// la convertir en chane de texte
		var prefString = prefsList.join("\n");
		// l'envoyer  la fonction qui crit le rsultat dans un fichier
		writeResult(prefString);
		// renvoyer le rsultat  l'appel de la fonction
		return [strWght, strTint, numVal, pcHeigh, pcWidth, dirDec];
	}
	else{
		// lors d'un clic sur "Annuler": supprimer le dialogue...
		aDialog.destroy();
		// ... appeler la fonction pour restaurer les rglages...
		reStoreSettings();
		// ... et arrter l'excution
		exit();
	}
}

// crire le rsultat dans un fichier /////////////////////////////////////////////////////////////
function writeResult(curPrefs) {
	if ($.os.slice(0,7) == "Windows") {
		var tempFolder = Folder.temp;
		var f = File(tempFolder + "/Spirale3D401.txt");
	}
	else {
		var f = File("~/Library/Preferences/Spirale3D401.txt");
	}
	var fileIsOpen = f.open("w");
	if (fileIsOpen) {
		try {
			f.write(curPrefs);
			f.close();
		}
		catch (e) {
			try {
				f.close();
			}
			catch (e) {
			}
		}
	}
}

// restaurer les rglages modifis ////////////////////////////////////////////////////////////////
function reStoreSettings() {
	// les units de mesure, l'origine des rgles, le point zro et le point de rfrence
	curDoc.viewPreferences.horizontalMeasurementUnits = hM;
	curDoc.viewPreferences.verticalMeasurementUnits = vM;
	curDoc.viewPreferences.rulerOrigin = rO;
	curDoc.zeroPoint = zP;
	curDoc.layoutWindows[0].transformReferencePoint = tR;
}